from src.fw_api import get_artists, get_tracks, get_audio_file
from src.fw_albums import list_albums
from src.mpv_control import player, player_menu, track_url_to_uuid
from pyfzf.pyfzf import FzfPrompt
from loguru import logger

fzf = FzfPrompt()


@logger.catch
def list_artists(pg=None, search=None):
    artists = get_artists(q=search, pg=pg)
    artists_next = artists.get('next')
    artists_prev = artists.get('previous')
    artists_results = artists.get('results')
    view = ['Search']
    if artists_next:
        view.append('Next page')
    if artists_prev:
        view.append('Prev page')

    for i in artists_results:
        index = artists_results.index(i)
        artist_name = i.get('name')
        view.append(f'{index}.{artist_name}')
    select = fzf.prompt(view)[0].split('.', 1)[0]
    if select == 'Next page':
        list_artists(pg=artists_next)
    elif select == 'Prev page':
        list_artists(pg=artists_prev)
    elif select == 'Search':
        print('Search by artist:')
        list_artists(search=input())
    else:
        list_albums(albums=artists_results[int(select)].get('albums'))


def play_artist(artist_id):
    tracks = get_tracks(artist=artist_id, include_channels=True, pg=None)
    tracks_next = tracks.get('next')
    tracks_count = tracks.get('count')
    storage = {}
    if tracks_count > 50:
        print(f'Loading {tracks_count} tracks...')
    elif tracks_count == 0:
        logger.warning('Empty tracks. Nothing to do')
        return
    while True:
        tracks_results = tracks.get('results')
        tracks_next = tracks.get('next')
        for i in tracks_results:
            listen_url = get_audio_file(i['listen_url'], True)
            storage[track_url_to_uuid(listen_url)] = i
            player.loadfile(listen_url, 'append-play')
        if tracks_next:
            tracks = get_tracks(
                artist=artist_id, include_channels=True, pg=tracks_next)
        else:
            break
    artist_name = tracks.get('results')[0]['artist']['name']
    player_menu(f"Artist {artist_name} playing...", storage)
